@@ -10,13 +10,8 @@ from typing import List, Deque, Callable
 
 import psutil
 import pytz
-from peek_abstract_chunked_index.private.server.controller.ACIProcessorStatusNotifierABC import \
-    ACIProcessorStatusNotifierABC
-from peek_abstract_chunked_index.private.tuples.ACIProcessorQueueTupleABC import \
-    ACIProcessorQueueTupleABC
-from peek_plugin_base.storage.RunPyInPg import runPyInPg
 from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT, ISOLATION_LEVEL_DEFAULT
-from sqlalchemy import asc, select
+from sqlalchemy import select, asc
 from sqlalchemy.dialects import postgresql
 from twisted.internet import task, reactor, defer
 from twisted.internet.defer import inlineCallbacks, Deferred, DeferredSemaphore
@@ -24,6 +19,12 @@ from vortex.DeferUtil import deferToThreadWrapWithLogger, vortexLogFailure, \
     nonConcurrentMethod
 from vortex.Payload import Payload
 
+from peek_abstract_chunked_index.private.server.controller.ACIProcessorStatusNotifierABC import \
+    ACIProcessorStatusNotifierABC
+from peek_abstract_chunked_index.private.tuples.ACIProcessorQueueTupleABC import \
+    ACIProcessorQueueTupleABC
+from peek_plugin_base.storage.RunPyInPg import runPyInPg
+
 ACIProcessorQueueBlockItem = namedtuple("ACIProcessorQueueBlockItem",
                                         ("queueIds", "itemsEncodedPayload",
                                          "itemUniqueIds"))
@@ -299,6 +300,8 @@ class ACIProcessorQueueControllerABC(metaclass=ABCMeta):
         toGrab = cls.QUEUE_BLOCKS_MAX - queueCount
         toGrab *= cls.QUEUE_ITEMS_PER_TASK
 
+        # This is sorted with the newest queue item first for "USE LAST ITEM"
+        # deduplication
         sql = select([queueTable]) \
             .order_by(asc(queueTable.c.id)) \
             .limit(toGrab)
@@ -333,20 +336,26 @@ class ACIProcessorQueueControllerABC(metaclass=ABCMeta):
         # ---------------
         # Process the queue items int blocks
 
+        # NOTE: Do not mess with this "USE LAST ITEM" deduplication strategy
+        # Some plugins include values in their queues and we want the latest
+        # value.
+        queueItems = list(reversed(queueItems))
+        addedUniqueKeys = set()
+
         queueBlocks = []
         for start in range(0, len(queueItems), cls.QUEUE_ITEMS_PER_TASK):
 
             queueIds = []
-            latestUpdates = {}
+            itemUniqueIds = []
+            items = []
             for item in queueItems[start: start + cls.QUEUE_ITEMS_PER_TASK]:
                 queueIds.append(item.id)
-                # NOTE: Do not mess with this "USE LAST ITEM" deduplication strategy
-                # Some plugins include values in their queues and we want the latest
-                # value.
-                latestUpdates[item.ckiUniqueKey] = item
+                if item.ckiUniqueKey in addedUniqueKeys:
+                    continue
 
-            itemUniqueIds = list(latestUpdates.keys())
-            items = list(latestUpdates.values())
+                addedUniqueKeys.add(item.ckiUniqueKey)
+                items.append(item)
+                itemUniqueIds.append(item.ckiUniqueKey)
 
             itemsEncodedPayload = Payload(tuples=[items, queueIds]).toEncodedPayload()
 
@@ -354,6 +363,9 @@ class ACIProcessorQueueControllerABC(metaclass=ABCMeta):
                 (queueIds, itemsEncodedPayload.decode(), itemUniqueIds)
             )
 
+        # Put the oldest queue blocks back at the start
+        queueBlocks = list(reversed(queueBlocks))
+
         return queueBlocks
 
     # ---------------
